#define USE_BLOB 1

/* Copyright (C) 2005, 2006 Hartmut Holzgraefe, MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include <mysql.h>
#include <NdbApi.hpp>

#include <time.h>

#define NOT_FOUND 626

char *key = NULL;

Ndb_cluster_connection *example_init(int *pargc, char ***pargv)
{
	char *connectstr = "localhost";
	int no_retries   = 4;
	int retry_delay  = 5; 
	int verbose      = 1;

	Ndb_cluster_connection *conn;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

	// import configuration 
	const char * mycnf_sections[] = {"mysql_cluster", NULL};
	load_defaults("my", mycnf_sections, pargc, pargv);

	// parse command line arguments
	struct option long_options[] = {
		{"connect_string", 1, 0, 'c'},
		{"try_reconnect" , 1, 0, 't'},
		{0, 0, 0, 0}
	};
	char *short_options = "c:";
	int opt, option_index;

	opterr = 0;
	do {
		opt = getopt_long(*pargc, *pargv, short_options, long_options, &option_index);
		switch (opt) {
		case 'c':
			connectstr = optarg;
			break;
		case 't':
			no_retries = atoi(optarg);
		default:
			break;
		}
	} while ( opt != -1);

	// prepare connection to cluster
	printf("trying to connect to '%s'\n", connectstr);
	conn = new Ndb_cluster_connection(connectstr);

	// initiate connection
	if (conn->connect(no_retries, retry_delay, verbose))
	{
		fprintf(stderr, "Unable to connect to cluster.");
		exit(EXIT_FAILURE);
	}
	
	// wait for the storage nodes (ndbd's)
	if (conn->wait_until_ready(30,0) < 0)
	{
		fprintf(stderr, "Cluster was not ready within 30 secs.\n");
		exit(EXIT_FAILURE);
	}

	return conn; 
}

void example_end(Ndb_cluster_connection *conn)
{
	// tear down connection
	delete conn;

	// shut down MySQL and Ndb client libraries
	ndb_end(0);
}



int main(int argc, char **argv)
{
    Ndb_cluster_connection *conn;
	char constr[1024];

	Uint64 bloblen;
	Uint32 readlen;
	
	char *val;
	Uint32 vallen;
	

	conn = example_init(&argc, &argv);

	if (argc > 1) {
	  key = (char *)calloc(33, 1);
	  sprintf(key, "%32.32s\n", argv[1]);
	  key[32] = '\0';
	} else {
	  printf("useage: delete [...] key\n");
	  return EXIT_FAILURE;
	}

	printf("test key is '%s'\n", key);

	Ndb *ndb = new Ndb(conn, "test");
	if (!ndb) {
	  fprintf(stderr, "Can't create NDB object\n");
	  exit(3);
	}
	if (ndb->init()) {
	  fprintf(stderr, "Can't create NDB object\n");
	  exit(3);
	}

	
	printf("get dictionary\n");
	const NdbDictionary::Dictionary* dict= ndb->getDictionary();
	if (NULL == dict) {
	  fprintf(stderr, "Can't get dictionary: %s\n", ndb->getNdbError().message);
	  return EXIT_FAILURE;
	}

	printf("lookup session table\n");
	const NdbDictionary::Table* ndb_table  = dict->getTable("session");
	if (NULL == ndb_table) {
	  fprintf(stderr, "Can't get table from dict: %s\n", dict->getNdbError().message);
	  return EXIT_FAILURE;
	}

	printf("start transaction\n");
	NdbTransaction * ndb_trans = ndb->startTransaction();
	if (NULL == ndb_trans) {
	  fprintf(stderr, "Can't get transaction: %s\n", ndb->getNdbError().message);
	  return EXIT_FAILURE;
	}

	printf("init operation\n");
	NdbOperation * ndb_delete_op = ndb_trans->getNdbOperation(ndb_table);
	if (NULL == ndb_delete_op) {
	  fprintf(stderr, "Can't get operation: %s\n", ndb_trans->getNdbError().message);
	  return EXIT_FAILURE;
	}

	printf("setting up for delete\n");
	if (0 > ndb_delete_op->deleteTuple()) {
	  fprintf(stderr, "Can't update tuple: %s\n", ndb_delete_op->getNdbError().message);
	  return EXIT_FAILURE;
	}
	
	printf("set up key lookup condition\n");
	if (0 < ndb_delete_op->equal("id", key)) {
	  fprintf(stderr, "equal failed: %s\n", ndb_delete_op->getNdbError().message);
	  return EXIT_FAILURE;
	}

	printf("execute\n");
	if (0 > ndb_trans->execute( NdbTransaction::NoCommit )) {
	  fprintf(stderr, "execute failed: %d %s\n", ndb_trans->getNdbError().code, ndb_trans->getNdbError().message);
	  return EXIT_FAILURE;    
	}

	printf("commit\n");
	if (0 > ndb_trans->execute( NdbTransaction::Commit )) {
	  fprintf(stderr, "commit failed: %d %s\n", ndb_trans->getNdbError().code, ndb_trans->getNdbError().message);
	  return EXIT_FAILURE;    
	}

	printf("cleanup\n");
	example_end(conn);

	printf("all fine!");
	
	return EXIT_SUCCESS;
}
