#define USE_BLOB 1

/* Copyright (C) 2005, 2006 Hartmut Holzgraefe, MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include <mysql.h>
#include <NdbApi.hpp>

#include <time.h>

#define NDBSESSION_G(x) x
#define php_error fprintf
#define E_WARNING stderr
#define FAILURE EXIT_FAILURE
#define emalloc malloc
#define ecalloc calloc

#define NOT_FOUND 626

const char *key = "33345678901234567890123456789012";

Ndb_cluster_connection *example_init(int *pargc, char ***pargv)
{
	char *connectstr = "localhost";
	int no_retries   = 4;
	int retry_delay  = 5; 
	int verbose      = 1;

	Ndb_cluster_connection *conn;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

	// import configuration 
	const char * mycnf_sections[] = {"mysql_cluster", NULL};
	load_defaults("my", mycnf_sections, pargc, pargv);

	// parse command line arguments
	struct option long_options[] = {
		{"connect_string", 1, 0, 'c'},
		{"try_reconnect" , 1, 0, 't'},
		{0, 0, 0, 0}
	};
	char *short_options = "c:";
	int opt, option_index;

	opterr = 0;
	do {
		opt = getopt_long(*pargc, *pargv, short_options, long_options, &option_index);
		switch (opt) {
		case 'c':
			connectstr = optarg;
			break;
		case 't':
			no_retries = atoi(optarg);
		default:
			break;
		}
	} while ( opt != -1);

	// prepare connection to cluster
	printf("trying to connect to '%s'\n", connectstr);
	conn = new Ndb_cluster_connection(connectstr);

	// initiate connection
	if (conn->connect(no_retries, retry_delay, verbose))
	{
		fprintf(stderr, "Unable to connect to cluster.");
		exit(EXIT_FAILURE);
	}
	
	// wait for the storage nodes (ndbd's)
	if (conn->wait_until_ready(30,0) < 0)
	{
		fprintf(stderr, "Cluster was not ready within 30 secs.\n");
		exit(EXIT_FAILURE);
	}

	return conn; 
}

void example_end(Ndb_cluster_connection *conn)
{
	// tear down connection
	delete conn;

	// shut down MySQL and Ndb client libraries
	ndb_end(0);
}



int main(int argc, char **argv)
{
    Ndb_cluster_connection *conn;
	char constr[1024];

	Uint64 bloblen;
	Uint32 readlen;
	
	char *val;
	Uint32 vallen;
	

	conn = example_init(&argc, &argv);

	Ndb *ndb = new Ndb(conn, "test");
	if (!ndb) {
	  fprintf(stderr, "Can't create NDB object\n");
	  exit(3);
	}
	if (ndb->init()) {
	  fprintf(stderr, "Can't create NDB object\n");
	  exit(3);
	}

	
	// get dictionary
	const NdbDictionary::Dictionary* dict= ndb->getDictionary();
	if (NULL == dict) {
	  fprintf(stderr, "Can't get dictionary: %s", ndb->getNdbError().message);
	  return EXIT_FAILURE;
	}

	// lookup session table
	const NdbDictionary::Table* ndb_table  = dict->getTable("session");
	if (NULL == ndb_table) {
	  fprintf(stderr, "Can't get table from dict: %s", dict->getNdbError().message);
	  return EXIT_FAILURE;
	}

	// start transaction
	NdbTransaction * ndb_trans = NDBSESSION_G(ndb)->startTransaction();
	if (NULL == NDBSESSION_G(ndb_trans)) {
	  php_error(E_WARNING, "Can't get transaction: %s", NDBSESSION_G(ndb)->getNdbError().message);
	  return FAILURE;
	}

	// === READ ==========================================================================
	
	// init operation
	NdbOperation * ndb_read_op = NDBSESSION_G(ndb_trans)->getNdbOperation(NDBSESSION_G(ndb_table));
	if (NULL == NDBSESSION_G(ndb_read_op)) {
	  php_error(E_WARNING, "Can't get operation: %s", NDBSESSION_G(ndb_trans)->getNdbError().message);
	  return FAILURE;
	}

	// setting up for read
	if (0 > NDBSESSION_G(ndb_read_op)->readTuple()) {
	  php_error(E_WARNING, "Can't update tuple: %s", NDBSESSION_G(ndb_read_op)->getNdbError().message);
	  return FAILURE;
	}
	
	// set up key lookup condition
	if (0 < NDBSESSION_G(ndb_read_op)->equal("id", key)) {
	  php_error(E_WARNING, "equal failed: %s", NDBSESSION_G(ndb_read_op)->getNdbError().message);
	  return FAILURE;
	}

#ifdef USE_BLOB
	// allocate blob
	NdbBlob * NDBSESSION_G(ndb_blob) = NDBSESSION_G(ndb_read_op)->getBlobHandle("data");
	if (NULL == NDBSESSION_G(ndb_blob)) {
	  php_error(E_WARNING, "can't get blob: %s", NDBSESSION_G(ndb_read_op)->getNdbError().message);
	  return FAILURE;
	}
#endif
 
	// execute without commit
	if (0 > NDBSESSION_G(ndb_trans)->execute( NdbTransaction::NoCommit )) {
	  if (NDBSESSION_G(ndb_trans)->getNdbError().code != NOT_FOUND) {
		php_error(E_WARNING, "can't execute read for '%s': %s", key, NDBSESSION_G(ndb_trans)->getNdbError().message);
		return FAILURE;
	  }
	  printf("no session entry yet\n");

	  // rollback and restart transaction
	  ndb_trans->close();

	  ndb_trans = NDBSESSION_G(ndb)->startTransaction();
	  if (NULL == NDBSESSION_G(ndb_trans)) {
		php_error(E_WARNING, "Can't get transaction: %s", NDBSESSION_G(ndb)->getNdbError().message);
		return FAILURE;
	  }

	  // init operation
	  NdbOperation * ndb_insert_op = NDBSESSION_G(ndb_trans)->getNdbOperation(NDBSESSION_G(ndb_table));
	  if (NULL == NDBSESSION_G(ndb_insert_op)) {
		php_error(E_WARNING, "Can't get operation: %s", NDBSESSION_G(ndb_trans)->getNdbError().message);
		return FAILURE;
	  }
	  
	  // we are going to update data later so this needs to be an update operation
	  if (0 > NDBSESSION_G(ndb_insert_op)->insertTuple()) {
		php_error(E_WARNING, "Can't insert tuple: %s", NDBSESSION_G(ndb_insert_op)->getNdbError().message);
		return FAILURE;
	  }

	  if (0 > ndb_insert_op->equal("id", key)) {
		php_error(E_WARNING, "equal failed: %s", NDBSESSION_G(ndb_insert_op)->getNdbError().message);
		return FAILURE;
	  }

	  if (0 > ndb_insert_op->setValue("usetime", (Int32)0)) {
		php_error(E_WARNING, "set value failed: %s", NDBSESSION_G(ndb_insert_op)->getNdbError().message);
		return FAILURE;
	  }

	  
	  // execute but don't commit yet
	  if (0 > NDBSESSION_G(ndb_trans)->execute( NdbTransaction::NoCommit )) {
		php_error(E_WARNING, "execute of insert failed: %s", NDBSESSION_G(ndb_trans)->getNdbError().message);
		return FAILURE;    
	  }

	  val = (char *)ecalloc(1, 1);
	  vallen = 0;
	} else {
	  // found
	  printf("reading session entry\n");

#ifdef USE_BLOB	  
	  // allocate blob data buffer
	  NDBSESSION_G(ndb_blob)->getLength(bloblen);
	  printf("bloblen %ld\n", (long)bloblen);
	  val = (char *)ecalloc((size_t)bloblen + 1, 1);

	  // TODO: this limits session data to 64k
	  readlen = bloblen;
	  if (0 < NDBSESSION_G(ndb_blob)->readData((void *)val, readlen)) {
		php_error(E_WARNING, "blob read failed");
		return FAILURE;
	  } 
	  php_error(E_WARNING, "blob data is (%d) '%s'\n", (int)bloblen, val);
	  vallen = readlen;	  
#endif
	}

	
	// ===  WRITE ================================================================

	// init operation
	NdbOperation * ndb_write_op = NDBSESSION_G(ndb_trans)->getNdbOperation(NDBSESSION_G(ndb_table));
	if (NULL == NDBSESSION_G(ndb_write_op)) {
	  php_error(E_WARNING, "Can't get operation: %s", NDBSESSION_G(ndb_trans)->getNdbError().message);
	  return FAILURE;
	}
	
	// we are going to update data later so this needs to be an update operation
	if (0 > NDBSESSION_G(ndb_write_op)->updateTuple()) {
	  php_error(E_WARNING, "Can't update tuple: %s", NDBSESSION_G(ndb_write_op)->getNdbError().message);
	  return FAILURE;
	}

	// set up key lookup condition
	if (0 < NDBSESSION_G(ndb_write_op)->equal("id", key)) {
	  php_error(E_WARNING, "equal failed: %s", NDBSESSION_G(ndb_write_op)->getNdbError().message);
	  return FAILURE;
	}

#ifdef USE_BLOB
	// allocate blob
	NdbBlob * NDBSESSION_G(ndb_write_blob) = NDBSESSION_G(ndb_write_op)->getBlobHandle("data");
	if (NULL == NDBSESSION_G(ndb_write_blob)) {
	  php_error(E_WARNING, "can't get blob: %s", NDBSESSION_G(ndb_write_op)->getNdbError().message);
	  return FAILURE;
	}
#endif

	const char *attr = "usetime";
	Int32 usetime = time(NULL);
	if (0  > ndb_write_op->setValue(attr, usetime)) {
		php_error(E_WARNING, "can't set value: %s", NDBSESSION_G(ndb_write_op)->getNdbError().message);
		return FAILURE;	  
	}

#ifdef USE_BLOB
	// execute but don't commit yet
	if (0 > NDBSESSION_G(ndb_trans)->execute( NdbTransaction::NoCommit )) {
	  php_error(E_WARNING, "exec failed");
	  return FAILURE;    
	}

	// change data: append another 'r'
	val[vallen] = 'r';
	vallen++;

	// truncate the blob before we put in new data 
	if (0 > NDBSESSION_G(ndb_write_blob)->truncate(vallen)) {
	  php_error(E_WARNING, "truncate failed");
	  return FAILURE;    
	}

	// rewind to the beginning of the blob
	if (0 > NDBSESSION_G(ndb_write_blob)->setPos(0)) {
	  php_error(E_WARNING, "setPos failed");
	  return FAILURE;    
	}

	// write data
	if (0 > NDBSESSION_G(ndb_write_blob)->writeData((void *)val, vallen)) {
	  php_error(E_WARNING, "write failed");
	  return FAILURE;    
	}
#endif

	// commit
	if (0 > NDBSESSION_G(ndb_trans)->execute( NdbTransaction::Commit )) {
	  php_error(E_WARNING, "commit failed");
	  return FAILURE;    
	}


	example_end(conn);

	printf("all fine!");
	
	return EXIT_SUCCESS;
}
